﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"

using namespace GraphicEqualizer2;

using namespace Concurrency;
using namespace Microsoft::WRL;
using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::Storage;
using namespace Windows::Storage::FileProperties;
using namespace Windows::Storage::Pickers;
using namespace Windows::Storage::Streams;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Automation;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

MainPage::MainPage()
{
	InitializeComponent();

    // Create an IXAudio2 object
    HRESULT hresult = XAudio2Create(&pXAudio2);

    if (FAILED(hresult))
        throw ref new COMException(hresult, "XAudio2Create failure");

    // Create a mastering voice
    hresult = pXAudio2->CreateMasteringVoice(&pMasteringVoice); 

    if (FAILED(hresult))
        throw ref new COMException(hresult, "IXAudio2->CreateMasteringVoice failure");

    // Initialize the volume
    hresult = pMasteringVoice->SetVolume(0);

    if (FAILED(hresult))
        throw ref new COMException(hresult, "IXAudio2MasteringVoice->SetVolume failure");

    // Create equalizer effect
    OneThirdOctaveEqualizerEffect * pEqualizerEffect = OneThirdOctaveEqualizerEffect::Create();

    // Reference the effect with two structures
    XAUDIO2_EFFECT_DESCRIPTOR filterEffectDescriptor;
    filterEffectDescriptor.pEffect = (IUnknown *) (IXAPOParameters *) pEqualizerEffect;
    filterEffectDescriptor.InitialState = true;
    filterEffectDescriptor.OutputChannels = 2;

    XAUDIO2_EFFECT_CHAIN filterEffectChain;
    filterEffectChain.EffectCount = 1;
    filterEffectChain.pEffectDescriptors = &filterEffectDescriptor;

    hresult = pMasteringVoice->SetEffectChain(&filterEffectChain);

    if (FAILED(hresult))
        throw ref new COMException(hresult, "pMasteringVoice->SetEffectChain failure");

    // And release this copy
    pEqualizerEffect->Release();

    // Define constants and array for creating equalizer sliders
    const int bandCount = 26;
    const float bandWidth = 1.0f / 3;       // in octaves

    static wchar_t* frequencies[26] = 
    {
        L"20", L"25", L"31.5", L"40", L"50", L"63", L"80", L"100", L"125",
        L"160", L"200", L"250", L"315", L"400", L"500", L"630", L"800", L"1K",
        L"1.25K", L"1.6K", L"2K", L"2.5K", L"3.15K", L"4K", L"5K", L"6.3K"
    };

    // Calculate Q (== 4.318)
    float Q = float(sqrt(pow(2, bandWidth)) / (pow(2, bandWidth) - 1));

    // Loop through the bands
    for (int i = 0; i < bandCount; i++)
    {
        // Create a Grid column
        ColumnDefinition^ coldef = ref new ColumnDefinition();
        coldef->Width = GridLengthHelper::FromValueAndType(1, GridUnitType::Star);
        equalizerSliderGrid->ColumnDefinitions->Append(coldef);

        // Create a label
        TextBlock^ txtblk = ref new TextBlock();
        txtblk->Text = ref new String(frequencies[i]);
        Grid::SetColumn(txtblk, i);
        Grid::SetRow(txtblk, 0);
        equalizerSliderGrid->Children->Append(txtblk);

        // Create a Slider
        Slider^ slider = ref new Slider();
        slider->Tag = i;
        slider->ValueChanged += ref new RangeBaseValueChangedEventHandler(this, &MainPage::OnEqualizerSliderChanged);
        Grid::SetColumn(slider, i);
        Grid::SetRow(slider, 1);
        equalizerSliderGrid->Children->Append(slider);

        // Another label
        txtblk = ref new TextBlock();
        txtblk->Text = ref new String(frequencies[i]);
        Grid::SetColumn(txtblk, i);
        Grid::SetRow(txtblk, 2);
        equalizerSliderGrid->Children->Append(txtblk);
    }
}

void MainPage::OnOpenButtonClick(Object^ sender, RoutedEventArgs^ args)
{
    FileOpenPicker^ fileOpenPicker = ref new FileOpenPicker();
    fileOpenPicker->SuggestedStartLocation = PickerLocationId::MusicLibrary;
    fileOpenPicker->FileTypeFilter->Append(".wma");
    fileOpenPicker->FileTypeFilter->Append(".mp3");
    fileOpenPicker->FileTypeFilter->Append(".wav");

    // Get a music file from the picker
    task<StorageFile^> pickSingleFileTask = create_task(fileOpenPicker->PickSingleFileAsync());
    pickSingleFileTask.then([this](StorageFile^ storageFile)
    {
        if (storageFile == nullptr)
            return;

        // Get the thumbnail for the display 
        task<StorageItemThumbnail^> getThumbnailTask = create_task(storageFile->GetThumbnailAsync(ThumbnailMode::SingleItem));
        getThumbnailTask.then([this](StorageItemThumbnail^ storageItemThumbnail)
        {
            task<void> setSourceTask(bitmapImage->SetSourceAsync(storageItemThumbnail));
            setSourceTask;
        });

        // Get information about the track
        StorageItemContentProperties^ contentProperties = storageFile->Properties;
        task<MusicProperties^> getMusicPropertiesTask = create_task(contentProperties->GetMusicPropertiesAsync());
        getMusicPropertiesTask.then([this](MusicProperties^ musicProperties)
        {
            artistText->Text = musicProperties->Artist;
            albumText->Text = musicProperties->Album;
            trackText->Text = musicProperties->Title;

            int durationInSeconds = int(musicProperties->Duration.Duration / 10000000);
            wchar_t str[16];
            swprintf(str, 16, L"%i:%02i", durationInSeconds / 60, 
                                          durationInSeconds % 60);
            durationText->Text = ref new String(str);
        });

        // Open the file for reading
        task<IRandomAccessStream^> openReadTask = create_task(storageFile->OpenAsync(FileAccessMode::Read));
        openReadTask.then([this](IRandomAccessStream^ randomAccessStream)
        {
            // Create an IMFSourceReader and an IXAudio2SourceVoice 
            if (pAudioFilePlayer != nullptr)
                delete pAudioFilePlayer;

            pAudioFilePlayer = new AudioFilePlayer(pXAudio2, pMasteringVoice, randomAccessStream);

            // Quite possibly start it playing
            if (pauseButton->Visibility == Windows::UI::Xaml::Visibility::Visible)
                pAudioFilePlayer->Start();
        });
    });
}

void MainPage::OnPlayButtonClick(Object^ sender, RoutedEventArgs^ args)
{
    if (pAudioFilePlayer != nullptr)
        pAudioFilePlayer->Start();

    playButton->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
    pauseButton->Visibility= Windows::UI::Xaml::Visibility::Visible;
}

void MainPage::OnPauseButtonClick(Object^ sender, RoutedEventArgs^ args)
{
    if (pAudioFilePlayer != nullptr)
        pAudioFilePlayer->Stop();

    playButton->Visibility = Windows::UI::Xaml::Visibility::Visible;
    pauseButton->Visibility= Windows::UI::Xaml::Visibility::Collapsed;
}

void MainPage::OnVolumeSliderValueChanged(Object^ sender, RangeBaseValueChangedEventArgs^ args)
{
    HRESULT hresult = pMasteringVoice->SetVolume(float(args->NewValue / 100));

    if (FAILED(hresult))
        ref new COMException(hresult, "IXAudio2MasteringVoice->SetVolume failure");
}

void MainPage::OnEqualizerInOutButtonClick(Object^ sender, RoutedEventArgs^ args)
{
    CheckBox^ checkBox = dynamic_cast<CheckBox^>(sender);

    if (checkBox->IsChecked->Value)
    {
        HRESULT hresult = pMasteringVoice->EnableEffect(0);

        if (FAILED(hresult))
            throw ref new COMException(hresult, "pMasteringVoice->EnableEffect failure");

        AutomationProperties::SetName(checkBox, "Equalizer In");
    }
    else
    {
        HRESULT hresult = pMasteringVoice->DisableEffect(0);

        if (FAILED(hresult))
            throw ref new COMException(hresult, "pMasteringVoice->DisableEffect failure");

        AutomationProperties::SetName(checkBox, "Bypass Equalizer");
    }
}

void MainPage::OnEqualizerSliderChanged(Platform::Object^ sender, RangeBaseValueChangedEventArgs^ args)
{
    Slider^ slider = dynamic_cast<Slider^>(sender);
    int bandIndex = (int)slider->Tag;
    float decibels = float(args->NewValue);
    equalizerParams.Amplitude[bandIndex] = float(pow(10, (decibels / 20)));
    HRESULT hresult = pMasteringVoice->SetEffectParameters(0, &equalizerParams, sizeof(equalizerParams));

    if (FAILED(hresult))
        throw ref new COMException(hresult, "pMasteringVoice->SetEffectParameters failure");
}

