﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"

using namespace SimpleEffectDemo;

using namespace Concurrency;
using namespace Microsoft::WRL;
using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::Storage;
using namespace Windows::Storage::FileProperties;
using namespace Windows::Storage::Pickers;
using namespace Windows::Storage::Streams;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Automation;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

MainPage::MainPage()
{
	InitializeComponent();

    // Create an IXAudio2 object
    HRESULT hresult = XAudio2Create(&pXAudio2);

    if (FAILED(hresult))
        throw ref new COMException(hresult, "XAudio2Create failure");

    // Create a mastering voice
    hresult = pXAudio2->CreateMasteringVoice(&pMasteringVoice); 

    if (FAILED(hresult))
        throw ref new COMException(hresult, "pXAudio2->CreateMasteringVoice failure");

    // Initialize the volume
    hresult = pMasteringVoice->SetVolume(0);

    if (FAILED(hresult))
        throw ref new COMException(hresult, "pMasteringVoice->SetVolume failure");

    // Create tremolo effect
    ComPtr<OneSecondTremoloEffect> pTremoloEffect = new OneSecondTremoloEffect();

    // Create echo effect
    ComPtr<OneSecondEchoEffect> pEchoEffect = new OneSecondEchoEffect();

    // Reference those effects with an effect descriptor array
    std::array<XAUDIO2_EFFECT_DESCRIPTOR, 2> effectDescriptors;

    effectDescriptors[0].pEffect = pTremoloEffect.Get();
    effectDescriptors[0].InitialState = tremoloToggle->IsChecked->Value;
    effectDescriptors[0].OutputChannels = 2;

    effectDescriptors[1].pEffect = pEchoEffect.Get();
    effectDescriptors[1].InitialState = echoToggle->IsChecked->Value;
    effectDescriptors[1].OutputChannels = 2;

    // Reference that array with an effect chain
    XAUDIO2_EFFECT_CHAIN effectChain;
    effectChain.EffectCount = effectDescriptors.size();
    effectChain.pEffectDescriptors = effectDescriptors.data();
    hresult = pMasteringVoice->SetEffectChain(&effectChain);

    if (FAILED(hresult))
        throw ref new COMException(hresult, "pMasteringVoice->SetEffectChain failure");
}

void MainPage::OnOpenButtonClick(Object^ sender, RoutedEventArgs^ args)
{
    FileOpenPicker^ fileOpenPicker = ref new FileOpenPicker();
    fileOpenPicker->SuggestedStartLocation = PickerLocationId::MusicLibrary;
    fileOpenPicker->FileTypeFilter->Append(".wma");
    fileOpenPicker->FileTypeFilter->Append(".mp3");
    fileOpenPicker->FileTypeFilter->Append(".wav");

    // Get a music file from the picker
    task<StorageFile^> pickSingleFileTask = create_task(fileOpenPicker->PickSingleFileAsync());
    pickSingleFileTask.then([this](StorageFile^ storageFile)
    {
        if (storageFile == nullptr)
            return;

        // Get the thumbnail for the display 
        task<StorageItemThumbnail^> getThumbnailTask = create_task(storageFile->GetThumbnailAsync(ThumbnailMode::SingleItem));
        getThumbnailTask.then([this](StorageItemThumbnail^ storageItemThumbnail)
        {
            task<void> setSourceTask(bitmapImage->SetSourceAsync(storageItemThumbnail));
            setSourceTask;
        });

        // Get information about the track
        StorageItemContentProperties^ contentProperties = storageFile->Properties;
        task<MusicProperties^> getMusicPropertiesTask = create_task(contentProperties->GetMusicPropertiesAsync());
        getMusicPropertiesTask.then([this](MusicProperties^ musicProperties)
        {
            artistText->Text = musicProperties->Artist;
            albumText->Text = musicProperties->Album;
            trackText->Text = musicProperties->Title;

            int durationInSeconds = int(musicProperties->Duration.Duration / 10000000);
            wchar_t str[16];
            swprintf(str, 16, L"%i:%02i", durationInSeconds / 60, 
                                          durationInSeconds % 60);
            durationText->Text = ref new String(str);
        });

        // Open the file for reading
        task<IRandomAccessStream^> openReadTask = create_task(storageFile->OpenAsync(FileAccessMode::Read));
        openReadTask.then([this](IRandomAccessStream^ randomAccessStream)
        {
            // Create an IMFSourceReader and an IXAudio2SourceVoice 
            if (pAudioFilePlayer != nullptr)
                delete pAudioFilePlayer;

            pAudioFilePlayer = new AudioFilePlayer(pXAudio2, pMasteringVoice, randomAccessStream);

            // Quite possibly start it playing
            if (pauseButton->Visibility == Windows::UI::Xaml::Visibility::Visible)
                pAudioFilePlayer->Start();
        });
    });
}

void MainPage::OnPlayButtonClick(Object^ sender, RoutedEventArgs^ args)
{
    if (pAudioFilePlayer != nullptr)
        pAudioFilePlayer->Start();

    playButton->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
    pauseButton->Visibility= Windows::UI::Xaml::Visibility::Visible;
}

void MainPage::OnPauseButtonClick(Object^ sender, RoutedEventArgs^ args)
{
    if (pAudioFilePlayer != nullptr)
        pAudioFilePlayer->Stop();

    playButton->Visibility = Windows::UI::Xaml::Visibility::Visible;
    pauseButton->Visibility= Windows::UI::Xaml::Visibility::Collapsed;
}

void MainPage::OnVolumeSliderValueChanged(Object^ sender, RangeBaseValueChangedEventArgs^ args)
{
    HRESULT hresult = pMasteringVoice->SetVolume(float(args->NewValue / 100));

    if (FAILED(hresult))
        ref new COMException(hresult, "IXAudio2MasteringVoice->SetVolume failure");
}

void MainPage::OnTremoloToggleChecked(Object^ sender, RoutedEventArgs^ args)
{
    EnableDisableEffect(safe_cast<ToggleButton^>(sender), 0);
}

void MainPage::OnEchoToggleChecked(Object^ sender, RoutedEventArgs^ args)
{
    EnableDisableEffect(safe_cast<ToggleButton^>(sender), 1);
}

void MainPage::EnableDisableEffect(ToggleButton^ toggle, int index)
{
    HRESULT hresult = toggle->IsChecked->Value ? pMasteringVoice->EnableEffect(index) :
                                                 pMasteringVoice->DisableEffect(index);
    if (FAILED(hresult))
        throw ref new COMException(hresult, "pMasteringVoice->Enable/DisableEffect " + 
                                            index.ToString());
}
