#include "pch.h"
#include "OneSecondTremoloEffect.h"

const XAPO_REGISTRATION_PROPERTIES OneSecondTremoloEffect::RegistrationProps = 
{
    __uuidof(OneSecondTremoloEffect),
    L"One-Second Tremolo Effect",
    L"Coded by Charles Petzold",
    1,      // major version number
    0,      // minor version number
    XAPOBASE_DEFAULT_FLAG | XAPO_FLAG_INPLACE_REQUIRED,
    1,      // min input buffer count
    1,      // max input buffer count
    1,      // min output buffer count
    1       // max output buffer count
};

HRESULT OneSecondTremoloEffect::LockForProcess(
                            UINT32 inpParamCount,
                            const XAPO_LOCKFORPROCESS_BUFFER_PARAMETERS  *pInpParams,
                            UINT32 outParamCount,
                            const XAPO_LOCKFORPROCESS_BUFFER_PARAMETERS  *pOutParams)
{
    waveFormat = * pInpParams[0].pFormat;

    return CXAPOBase::LockForProcess(inpParamCount, pInpParams, 
                                     outParamCount, pOutParams);
}

void OneSecondTremoloEffect::Process(UINT32 inpParamCount,
                                     const XAPO_PROCESS_BUFFER_PARAMETERS *pInpParams,
                                     UINT32 outParamCount,
                                     XAPO_PROCESS_BUFFER_PARAMETERS *pOutParams,
                                     BOOL isEnabled)
{
    XAPO_BUFFER_FLAGS flags = pInpParams[0].BufferFlags;
    int frameCount = pInpParams[0].ValidFrameCount;
    const float * pSrc = static_cast<float *>(pInpParams[0].pBuffer);
    float * pDst = static_cast<float *>(pOutParams[0].pBuffer);
    int numChannels = waveFormat.nChannels;

    switch(flags)
    {
    case XAPO_BUFFER_VALID:
        for (int frame = 0; frame < frameCount; frame++)
        {
            float sin = 1;

            if (isEnabled)
            {
                sin = fabs(DirectX::XMScalarSin(DirectX::XM_PI * tremoloIndex / 
                                                        waveFormat.nSamplesPerSec));
                tremoloIndex = (tremoloIndex + 1) % waveFormat.nSamplesPerSec;
            }

            for (int channel = 0; channel < numChannels; channel++)
            {
                int index = numChannels * frame + channel;
                pDst[index] = sin * pSrc[index];
            }
        }
        break;

    case XAPO_BUFFER_SILENT:
        break;
    }

    pOutParams[0].ValidFrameCount = pInpParams[0].ValidFrameCount;
    pOutParams[0].BufferFlags = pInpParams[0].BufferFlags;
}





